@@ -1,3 +1,5 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
 #
 #  FreeType high-level python API - Copyright 2011 Nicolas P. Rougier
@@ -38,8 +40,49 @@ __handle__ = None
 FT_Init_FreeType       = __dll__.FT_Init_FreeType
 FT_Done_FreeType       = __dll__.FT_Done_FreeType
 FT_Library_Version     = __dll__.FT_Library_Version
+
+def __del_library__(self):
+    global __handle__
+    if __handle__:
+        try:
+            FT_Done_FreeType(byref(self))
+            __handle__ = None
+        except:
+            pass
+FT_Library.__del__ = __del_library__
+
+def get_handle():
+    '''
+    Get unique FT_Library handle
+    '''
+    global __handle__
+    if not __handle__:
+        __handle__ = FT_Library( )
+        error = FT_Init_FreeType( byref(__handle__) )
+        if error: raise FT_Exception(error)
+        try:
+            set_lcd_filter( FT_LCD_FILTER_DEFAULT )
+        except:
+            pass
+        if error: raise FT_Exception(error)
+    return __handle__
+
+def version():
+    '''
+    Return the version of the FreeType library being used as a tuple of
+    ( major version number, minor version number, patch version number )
+    '''
+    amajor = FT_Int()
+    aminor = FT_Int()
+    apatch = FT_Int()
+    library = get_handle()
+    FT_Library_Version(library, byref(amajor), byref(aminor), byref(apatch))
+    return (amajor.value, aminor.value, apatch.value)
+
+
 FT_Library_SetLcdFilter= __dll__.FT_Library_SetLcdFilter
-FT_Library_SetLcdFilterWeights = __dll__.FT_Library_SetLcdFilterWeights
+if version()>=(2,4,0):
+    FT_Library_SetLcdFilterWeights = __dll__.FT_Library_SetLcdFilterWeights
 FT_New_Face            = __dll__.FT_New_Face
 FT_New_Memory_Face     = __dll__.FT_New_Memory_Face
 FT_Open_Face           = __dll__.FT_Open_Face
@@ -60,6 +103,9 @@ FT_Get_Kerning         = __dll__.FT_Get_Kerning
 FT_Get_Track_Kerning   = __dll__.FT_Get_Track_Kerning
 FT_Get_Glyph_Name      = __dll__.FT_Get_Glyph_Name
 FT_Get_Glyph           = __dll__.FT_Get_Glyph
+
+FT_Glyph_Get_CBox      = __dll__.FT_Glyph_Get_CBox
+
 FT_Get_Postscript_Name = __dll__.FT_Get_Postscript_Name
 FT_Get_Postscript_Name.restype = c_char_p
 FT_Select_Charmap      = __dll__.FT_Select_Charmap
@@ -78,6 +124,8 @@ FT_Get_Sfnt_Name       = __dll__.FT_Get_Sfnt_Name
 
 FT_Outline_GetInsideBorder  = __dll__.FT_Outline_GetInsideBorder
 FT_Outline_GetOutsideBorder = __dll__.FT_Outline_GetOutsideBorder
+FT_Outline_Get_BBox         = __dll__.FT_Outline_Get_BBox
+FT_Outline_Get_CBox         = __dll__.FT_Outline_Get_CBox
 FT_Stroker_New              = __dll__.FT_Stroker_New
 FT_Stroker_Set              = __dll__.FT_Stroker_Set
 FT_Stroker_Rewind           = __dll__.FT_Stroker_Rewind
@@ -98,54 +146,9 @@ FT_Glyph_To_Bitmap          = __dll__.FT_Glyph_To_Bitmap
 
 
 
-# -----------------------------------------------------------------------------
-# High-level python API 
-# -----------------------------------------------------------------------------
-def __del_library__(self):
-    global __handle__
-    if __handle__:
-        try:
-            FT_Done_FreeType(byref(self))
-            __handle__ = None
-        except:
-            pass
-FT_Library.__del__ = __del_library__
-
-
-def get_handle():
-    '''
-    Get unique FT_Library handle
-    '''
-    global __handle__
-    if not __handle__:
-        __handle__ = FT_Library( )
-        error = FT_Init_FreeType( byref(__handle__) )
-        if error: raise FT_Exception(error)
-        try:
-            set_lcd_filter( FT_LCD_FILTER_DEFAULT )
-        except:
-            pass
-        if error: raise FT_Exception(error)
-    return __handle__
-
-
-
 # -----------------------------------------------------------------------------
 #  Stand alone functions
 # -----------------------------------------------------------------------------
-def version():
-    '''
-    Return the version of the FreeType library being used as a tuple of
-    ( major version number, minor version number, patch version number )
-    '''
-    amajor = FT_Int()
-    aminor = FT_Int()
-    apatch = FT_Int()
-    library = get_handle()
-    FT_Library_Version(library, byref(amajor), byref(aminor), byref(apatch))
-    return (amajor.value, aminor.value, apatch.value)
-
-
 def set_lcd_filter(filt):
     '''
     This function is used to apply color filtering to LCD decimated bitmaps,
@@ -190,11 +193,19 @@ def set_lcd_filter_weights(a,b,c,d,e):
     FT_Library_SetLcdFilter. By default, FreeType uses the quintuple (0x00,
     0x55, 0x56, 0x55, 0x00) for FT_LCD_FILTER_LIGHT, and (0x10, 0x40, 0x70,
     0x40, 0x10) for FT_LCD_FILTER_DEFAULT and FT_LCD_FILTER_LEGACY.
+
+    Note:
+    -----
+    Only available if version > 2.4.0
     '''
-    library = get_handle()
-    weights = FT_Char(5)(a,b,c,d,e)
-    error = FT_Library_SetLcdFilterWeights(library, weights)
-    if error: raise FT_Exception(error)
+    if version()>=(2,4,0):
+        library = get_handle()
+        weights = FT_Char(5)(a,b,c,d,e)
+        error = FT_Library_SetLcdFilterWeights(library, weights)
+        if error: raise FT_Exception(error)
+    else:
+        raise RuntimeError, \
+              'set_lcd_filter_weights require freetype > 2.4.0'
 
 
 
@@ -620,6 +631,37 @@ class Outline( object ):
         '''
         return FT_Outline_GetInsideBorder( self._FT_Outline )
 
+    def get_bbox(self):
+        '''
+        Compute the exact bounding box of an outline. This is slower than
+        computing the control box. However, it uses an advanced algorithm which
+        returns very quickly when the two boxes coincide. Otherwise, the
+        outline Bezier arcs are traversed to extract their extrema.
+        '''
+        bbox = FT_BBox()
+        error = FT_Outline_Get_BBox(byref(self._FT_Outline), byref(bbox))
+        if error: raise FT_Exception(error)
+        return bbox
+
+    def get_cbox(self):
+        '''
+        Return an outline's 'control box'. The control box encloses all the
+        outline's points, including Bezier control points. Though it coincides
+        with the exact bounding box for most glyphs, it can be slightly larger
+        in some situations (like when rotating an outline which contains Bezier
+        outside arcs).
+
+        Computing the control box is very fast, while getting the bounding box
+        can take much more time as it needs to walk over all segments and arcs
+        in the outline. To get the latter, you can use the 'ftbbox' component
+        which is dedicated to this single task.
+        '''
+        bbox = FT_BBox()
+        error = FT_Outline_Get_CBox(byref(self._FT_Outline), byref(bbox))
+        if error: raise FT_Exception(error)
+        return BBox(bbox)
+
+
 
 
 # -----------------------------------------------------------------------------
@@ -702,6 +744,63 @@ class Glyph( object ):
         if error: raise FT_Exception( error )
         return BitmapGlyph( self._FT_Glyph )
 
+    def get_cbox(self, bbox_mode):
+        '''
+        Return an outline's 'control box'. The control box encloses all the
+        outline's points, including Bezier control points. Though it coincides
+        with the exact bounding box for most glyphs, it can be slightly larger
+        in some situations (like when rotating an outline which contains Bezier
+        outside arcs).
+
+        Computing the control box is very fast, while getting the bounding box
+        can take much more time as it needs to walk over all segments and arcs
+        in the outline. To get the latter, you can use the 'ftbbox' component
+        which is dedicated to this single task.
+
+        Parameters:
+        -----------
+
+        mode : The mode which indicates how to interpret the returned bounding
+               box values.
+
+        Note:
+        -----
+
+        Coordinates are relative to the glyph origin, using the y upwards
+        convention.
+
+        If the glyph has been loaded with FT_LOAD_NO_SCALE, 'bbox_mode' must be
+        set to FT_GLYPH_BBOX_UNSCALED to get unscaled font units in 26.6 pixel
+        format. The value FT_GLYPH_BBOX_SUBPIXELS is another name for this
+        constant.
+
+        Note that the maximum coordinates are exclusive, which means that one
+        can compute the width and height of the glyph image (be it in integer
+        or 26.6 pixels) as:
+
+        width  = bbox.xMax - bbox.xMin;                                  
+        height = bbox.yMax - bbox.yMin;                                  
+
+        Note also that for 26.6 coordinates, if 'bbox_mode' is set to
+        FT_GLYPH_BBOX_GRIDFIT, the coordinates will also be grid-fitted, which
+        corresponds to:
+
+        bbox.xMin = FLOOR(bbox.xMin);
+        bbox.yMin = FLOOR(bbox.yMin);                                    
+        bbox.xMax = CEILING(bbox.xMax);                                  
+        bbox.yMax = CEILING(bbox.yMax);                                  
+        
+        To get the bbox in pixel coordinates, set 'bbox_mode' to
+        FT_GLYPH_BBOX_TRUNCATE.
+
+        To get the bbox in grid-fitted pixel coordinates, set 'bbox_mode' to
+        FT_GLYPH_BBOX_PIXELS.
+        '''
+        bbox = FT_BBox()
+        error = FT_Glyph_Get_CBox(byref(self._FT_Glyph), bbox_mode,byref(bbox))
+        if error: raise FT_Exception(error)
+        return BBox(bbox)
+
 
 
 # -----------------------------------------------------------------------------
@@ -881,6 +980,12 @@ class Face( object ):
         self._index = index
         self._FT_Face = face
     
+    def __del__( self ):
+        '''
+        Discard  face object, as well as all of its child slots and sizes.
+        '''
+        FT_Done_Face( self._FT_Face )
+
     def set_char_size( self, width=0, height=0, hres=72, vres=72 ):
         '''
         This function calls FT_Request_Size to request the nominal size (in
